#!/usr/bin/env perl

use Test::More tests => 1;

BEGIN {
    use_ok('Catalyst::TraitFor::Request::Params::Hashed');
}

diag("Testing Catalyst::TraitFor::Request::Params::Hashed $Catalyst::TraitFor::Request::Params::Hashed::VERSION, Perl $], $^X");
